use std::sync::mpsc::{Receiver, Sender};

use super::super::init::config;
use std::collections::HashMap;
use std::convert::TryInto;
use std::net::Ipv4Addr;
use std::sync::Arc;
use tun_tap::Iface;

pub fn tun_recv_thread(
    config: Arc<config::Config>,
    iface: Arc<Iface>,
    sender_vec: Vec<Sender<super::TunData>>,
) {
    let mut ip_to_thread: HashMap<Ipv4Addr, u32> = HashMap::new();
    for (i, elt) in config.peers.iter().enumerate() {
        ip_to_thread.insert(elt.ip, i as u32);
    }

    loop {
        let mut buf = vec![0; super::TUN_PACKET_SIZE];
        let long = match iface.recv(&mut buf) {
            Ok(l) => l,
            Err(e) => {
                println!("{:?}", e);
                0
            }
        };
        // min ipv4 header size
        if long > 20 {
            let ip_dest = Ipv4Addr::new(buf[16], buf[17], buf[18], buf[19]);
            let thread_id: usize = (*ip_to_thread.get(&ip_dest).unwrap()).try_into().unwrap();
            sender_vec[thread_id]
                .send(super::TunData { payload: buf })
                .unwrap();
        }
    }
}

pub fn tun_send_thread(iface: Arc<Iface>, recv: Receiver<Vec<u8>>) {}
